/*
    Copyright 2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "aes.h"

#ifdef WIN32
    #include <windows.h>
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif



#define VER         "0.1"
#define BUFFSZ      4096
#define KEYPATH     "Software\\FTPWare\\COREFTP"


#ifdef WIN32
int regkey(HKEY hKey, LPCTSTR lpSubKey, LPTSTR lpValueName, uint8_t *buff, int len);
int enum_key(HKEY hkey, uint8_t *reg_path);
int enum_value(HKEY hkey, uint8_t *reg_path);
#endif
void coreftppwd(uint8_t *pwd);



int main(int argc, char *argv[]) {
    uint8_t buff[BUFFSZ + 1];

    setbuf(stdout, NULL);

    fputs("\n"
        "CoreFTP password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <password>\n"
            "\n"
#ifdef WIN32
            "Passwords in the registry are automatically showed below if no argument passed:\n"
#endif
            "\n", argv[0]);
#ifdef WIN32
        enum_key(HKEY_CURRENT_USER,  KEYPATH);
        enum_key(HKEY_USERS,         KEYPATH);
        enum_key(HKEY_LOCAL_MACHINE, KEYPATH);
#endif
        goto quit;
    }

    sprintf(buff, "%.*s", BUFFSZ, argv[1]);
    coreftppwd(buff);

quit:
    printf("\n  Press RETURN to exit\n");
    fgetc(stdin);
    return(0);
}



#ifdef WIN32

int regkey(HKEY hKey, LPCTSTR lpSubKey, LPTSTR lpValueName, uint8_t *buff, int len) {
    HKEY    key;

    buff[0] = 0;

    if(RegOpenKeyEx(hKey, lpSubKey, 0, KEY_READ, &key) != ERROR_SUCCESS) {
        return(-1);
    }
    if(RegQueryValueEx(key, lpValueName, NULL, NULL, buff, (void *)&len) != ERROR_SUCCESS) {
        RegCloseKey(key);
        return(-1);
    }
    RegCloseKey(key);


    /* CODE ADDED FOR coreftppwd */
    if(!stricmp(lpValueName, "host")) {
        printf("\n- host:     %s\n", buff);
    } else if(!stricmp(lpValueName, "user")) {
        printf("- username: %s\n", buff);
    } else if(stristr(lpValueName, "PW")) {
        coreftppwd(buff);
    }
    /* END */

    return(0);
}



int enum_key(HKEY hkey, uint8_t *reg_path) {
    HKEY    key;
    uint8_t buff[BUFFSZ + 1],
            *p;
    int     len,
            plen,
            k = 0;

    if(RegOpenKeyEx(hkey, reg_path, 0, KEY_READ, &key)) {
        return(-1);
    }

    plen = snprintf(
        buff,
        BUFFSZ,
        "%s\\",
        reg_path);
    if((plen < 0) || (plen >= BUFFSZ)) {
        return(-1);
    }

    enum_value(hkey, buff);

    p = buff + plen;

    for(;;) {
        len = BUFFSZ - plen;
        if(RegEnumKeyEx(
            key,
            k++,
            p,
            (void *)&len,
            NULL,
            NULL,
            NULL,
            NULL)) break;

        enum_key(hkey, buff);
    }

    RegCloseKey(key);
    return(0);
}



int enum_value(HKEY hkey, uint8_t *reg_path) {
    HKEY    key;
    uint8_t buff[BUFFSZ + 1],
            value[BUFFSZ + 1];
    int     len,
            k = 0;

    if(RegOpenKeyEx(hkey, reg_path, 0, KEY_READ, &key)) {
        return(-1);
    }

    for(;;) {
        len = BUFFSZ;
        if(RegEnumValue(
            key,
            k++,
            buff,
            (void *)&len,
            NULL,
            NULL,
            NULL,
            NULL)) break;

        regkey(hkey, reg_path, buff, value, sizeof(value));
    }

    RegCloseKey(key);
    return(0);
}

#endif



void coreftppwd(uint8_t *pwd) {
    aes_context ctx;
    int     len,
            tmp;
    uint8_t pwdkey[16] = "hdfzpysvpzimorhk",
//            unkkey[16] = "egj|pynqsygonshk",    // used for what?
            *p,
            *o;

    for(p = o = pwd; *p; p += 2, o++) {
        sscanf(p, "%02x", &tmp);
        *o = tmp;
    }
    *o = 0; // needed?
    len = o - pwd;
    len >>= 4;

    if(!len) return;

    aes_set_key(&ctx, pwdkey, sizeof(pwdkey) << 3);

    for(p = pwd; len--; p += 16) {
        aes_decrypt(&ctx, p, p);
    }

    printf("- password: %s\n", pwd);
}



